#!/usr/bin/env Rscript --vanilla

# Plot coverage histograms generated by e.g. 'mccortex31 clean --covg-before out.csv ...'
# input csv should have the columns: 'Covg' and 'NumUnitigs'
#
args <- commandArgs(trailingOnly=TRUE)
if(length(args) != 4) {
  stop("Usage: Rscript --vanilla plot-covg-hist.R <covg.csv> <out.pdf> [<cutoff> [<kcov>]]\n  Optional: set cutoff, kcov to zero\n")
}

cutoff <- 0
kcov <- 0

input_csv <- args[1]
output_pdf <- args[2]
if(length(args) >= 3) { cutoff <- as.numeric(args[3]) }
if(length(args) >= 4) { kcov <- as.numeric(args[4]) }

library('ggplot2')
library('gridExtra')

d <- read.csv(file=input_csv,sep=',',as.is=T)

ymax <- max(d[,'NumKmers'],d[,'NumUnitigs'])
cutline <- data.frame(x=as.numeric(c(cutoff,cutoff)), y=as.numeric(c(0,ymax)))
kcovline <- data.frame(x=as.numeric(c(kcov,kcov)), y=as.numeric(c(0,ymax)))

p <- ggplot() +
       geom_bar(data=d, stat="identity", aes(x=Covg,y=NumKmers,fill=FALSE), colour="firebrick") +
       guides(fill=FALSE) +
       xlab("Coverage (X)") +
       ylab("Number of kmers") +
       ggtitle("Kmer Coverage") +
       xlim(0,150)

if(cutoff > 0) {
  cutline[,'y'] = c(0, max(d[,'NumKmers']))
  p <- p + geom_line(data=cutline, aes(x=x,y=y), colour="black")
}

if(kcov > 0) {
  kcovline[,'y'] = c(0, max(d[,'NumKmers']))
  p <- p + geom_line(data=kcovline, aes(x=x,y=y), colour="black", linetype="dashed")
}

q <- ggplot() +
       geom_bar(data=d, stat="identity", aes(x=Covg,y=NumUnitigs,fill=FALSE), colour="firebrick") +
       guides(fill=FALSE) +
       xlab("Coverage (X)") +
       ylab("Number of unitigs") +
       ggtitle("Unitig Median Coverage") +
       xlim(0,150)

if(cutoff > 0) {
  cutline[,'y'] = c(0, max(d[,'NumUnitigs']))
  q <- q + geom_line(data=cutline, aes(x=x,y=y), colour="black")
}

if(kcov > 0) {
  kcovline[,'y'] = c(0, max(d[,'NumUnitigs']))
  q <- q + geom_line(data=kcovline, aes(x=x,y=y), colour="black", linetype="dashed")
}

# should work but doesn't
#pq <- arrangeGrob(p, q, ncol=2)
#ggsave(pq, file=output_pdf, width=6, height=6)
# Instead:
# pq <- arrangeGrob(p, q, ncol=2)
# put_pdf,width=12,height=6)
# grid.arrange(pq, ncol=1)
# dev.off

pdf(output_pdf,width=12,height=6)
grid.arrange(p, q, ncol=2)
dev.off()
