#!/usr/bin/env Rscript --vanilla

# Plot coverage matrix generated by e.g. 'mccortex31 links --covg-hist out.csv ...'
#
args <- commandArgs(trailingOnly=TRUE)
if(length(args) < 2 || length(args) > 4) {
  stop("Usage: ./link-cov-heatmap.R <covg.csv> <out.pdf> [maxdist [linear|log]]\n")
}

maxdist=0
axis_arg='linear'

input_csv <- args[1]
output_pdf <- args[2]

if(length(args) >= 3) { maxdist <- as.numeric(args[3]) }
if(length(args) >= 4) { axis_arg <- args[4] }

use_y_log = (axis_arg == 'log')

library('ggplot2')
library('reshape')
library('scales')
library('plyr')

cat("input_csv='",input_csv,"'\n",sep='')
cat("output_pdf='",output_pdf,"'\n",sep='')
cat('maxdist=',maxdist,'\n',sep='')

r <- read.table(input_csv,sep=',',head=T,row.names=1,comment.char='#',as.is=T)

if(maxdist == 0) { maxdist=nrow(r) }
maxdist<-min(nrow(r), maxdist)

if(nrow(r) > maxdist) {
  r<-r[1:maxdist,]
}

r<-as.matrix(r)

# Convert row/column names to numbers
colnames(r) <- 1:ncol(r)
rownames(r) <- 1:nrow(r)

sum.times <- function(x) {
  s <- 0
  for(i in 1:length(x)) { s <- s + i*x[i] }
  return(s)
}

counts <- data.frame(dist=1:maxdist, counts=apply(r, 1, sum.times))

get_step <- function(max) {
  s <- floor((max+99)/100)*10
  if(s == 0) { return(1); }
  else { return(s); }
}

maxcov<-max(counts$counts)

xstep<-get_step(maxdist)
xticks=seq(0,maxdist,xstep)
ystep<-get_step(maxcov)
yticks=seq(0,maxcov,ystep)

p <- ggplot(counts, aes(x=dist, y=counts)) + geom_line() +
     xlab("Link length in kmers")

if(use_y_log) {
  p <- p + ylab("Count (log)") + scale_y_log10()
} else {
  p <- p + ylab("Count")
}

ggsave(p, file=output_pdf, width=6, height=6)
