#!/usr/bin/env Rscript --vanilla

# Plot coverage matrix generated by e.g. 'mccortex31 links --covg-hist out.csv ...'
#
args <- commandArgs(trailingOnly=TRUE)
if(length(args) < 2 || length(args) > 5) {
  stop("Usage: ./plot-link-dist-cov.R <covg.csv> <out.pdf> [dist [cutoff [maxcov]]]\n")
}

dist=1
cutoff=0
maxcov=0

input_csv <- args[1]
output_pdf <- args[2]

if(length(args) >= 3) { dist <- as.numeric(args[3]) }
if(length(args) >= 4) { cutoff <- as.numeric(args[4]) }
if(length(args) >= 5) { maxcov <- as.numeric(args[5]) }

library('ggplot2')
library('reshape')
library('scales')
library('plyr')

cat("input_csv='",input_csv,"'\n",sep='')
cat("output_pdf='",output_pdf,"'\n",sep='')
cat('dist=',dist,'\n',sep='')
cat('maxcov=',maxcov,'\n',sep='')

r <- read.table(input_csv,sep=',',head=T,row.names=1,comment.char='#',as.is=T)

if(maxcov == 0) { maxcov=ncol(r) }
maxcov<-min(ncol(r), maxcov)
r <- r[,1:maxcov]

v<-as.numeric(r[dist,])
d<-data.frame(x=1:maxcov, y=v)

p <- ggplot(d, aes(x=x, y=y)) + geom_line() +
     xlab("Link coverage") +
     ylab(paste("Number of links of length",(dist+1),"(kmers)")) +
     geom_vline(xintercept=cutoff, color="red")


ggsave(p, file=output_pdf, width=6, height=6)
